@@ -618,6 +618,7 @@ gfuji\100cpan.org                       g.psy.va\100gmail.com
 gerard\100ggoossen.net                  gerard\100tty.nl
 gibreel\100pobox.com                    stephen.zander\100interlock.mckesson.com
 +                                       srz\100loopback
+gideon\100cpan.org                      gidisrael\100gmail.com
 gnat\100frii.com                        gnat\100prometheus.frii.com
 gp\100familiehaase.de                   gerrit\100familiehaase.de
 grazz\100pobox.com                      grazz\100nyc.rr.com
@@ -659,6 +660,8 @@ jhannah\100mutationgrid.com             jay\100jays.net
 +                                       jhannah\100omnihotels.com
 jidanni\100jidanni.org                  jidanni\100hoffa.dreamhost.com
 jjore\100cpan.org                       twists\100gmail.com
+jkeenan\100cpan.org                     jkeen\100verizon.net
++                                       jkeenan\100dromedary-001.ams6.corp.booking.com
 jns\100integration-house.com            jns\100gellyfish.com
 +                                       gellyfish\100gellyfish.com
 john\100atlantech.com                   john\100titanic.atlantech.com
@@ -672,6 +675,7 @@ jpeacock\100messagesystems.com          john.peacock\100havurah-software.org
 +                                       jpeacock\100jpeacock-hp.doesntexist.org
 +                                       jpeacock\100cpan.org
 +                                       jpeacock\100rowman.com
+jpl.jpl\100gmail.com                    jpl\100research.att.com
 jql\100accessone.com                    jql\100jql.accessone.com
 jsm28\100hermes.cam.ac.uk               jsm28\100cam.ac.uk
 
@@ -683,6 +687,7 @@ kane\100dwim.org                        kane\100xs4all.net
 keith.s.thompson\100gmail.com           kst\100mib.org
 ken\100mathforum.org                    kenahoo\100gmail.com
 +                                       ken.williams\100thomsonreuters.com
+kentfredric\100gmail.com                kentnl\100cpan.org
 kroepke\100dolphin-services.de          kay\100dolphin-services.de
 kst\100mib.org                          kst\100cts.com
 +                                       kst\100SDSC.EDU
@@ -919,4 +924,3 @@ wolfgang.laun\100alcatel.at             wolfgang.laun\100chello.at
 wolfsage\100gmail.com                   mhorsfall\100darmstadtium.(none)
 yath\100yath.de                         yath-perlbug\100yath.de
 
-jkeen@verizon.net                       jkeenan@cpan.org
