@@ -10,7 +10,7 @@ use File::Path ();
 use File::Spec ();
 use CPAN::Mirrors ();
 use vars qw($VERSION $auto_config);
-$VERSION = "5.5303";
+$VERSION = "5.5304";
 
 =head1 NAME
 
@@ -420,6 +420,14 @@ host should be tried first.
 
 Randomize parameter
 
+=item recommends_policy
+
+(Experimental feature!) Some CPAN modules recommend additional, optional dependencies.  These should
+generally be installed except in resource constrained environments.  When this
+policy is true, recommended modules will be included with required modules.
+
+Included recommended modules?
+
 =item scan_cache
 
 By default, each time the CPAN module is started, cache scanning is
@@ -462,6 +470,14 @@ variable.
 
 Show all individual modules that have a $VERSION of zero?
 
+=item suggests_policy
+
+(Experimental feature!) Some CPAN modules suggest additional, optional dependencies.  These 'suggest'
+dependencies provide enhanced operation.  When this policy is true, suggested
+modules will be included with required modules.
+
+Included suggested modules?
+
 =item tar_verbosity
 
 When CPAN.pm uses the tar command, which switch for the verbosity
@@ -865,6 +881,8 @@ sub init {
                    'follow|ask|ignore');
     my_prompt_loop(build_requires_install_policy => 'yes', $matcher,
                    'yes|no|ask/yes|ask/no');
+    my_yn_prompt(recommends_policy => 1, $matcher);
+    my_yn_prompt(suggests_policy => 0, $matcher);
 
     #
     #= Module::Signature
@@ -1029,7 +1047,7 @@ sub init {
         and $^O ne "MSWin32") {
         # as long as Windows needs $self->_build_command, we cannot
         # support sudo on windows :-)
-        my $default = "./Build";
+        my $default = $^O eq 'VMS' ? '@Build.com' : "./Build";
         if ( $CPAN::Config->{install_help} eq 'sudo' ) {
             if ( find_exe('sudo') ) {
                 $default = "sudo $default";
@@ -1224,10 +1242,7 @@ sub init {
             );
         }
         else {
-            $CPAN::Frontend->myprint(
-                "Autoconfigured everything but 'urllist'.\n"
-            );
-            _do_pick_mirrors();
+            $CPAN::Config->{urllist} = [ 'http://www.cpan.org/' ];
         }
     }
     elsif (!$matcher || "urllist" =~ $matcher) {
@@ -1393,6 +1408,8 @@ sub _do_pick_mirrors {
     my $_conf = 'n';
     if ( $CPAN::META->has_usable("Net::Ping") && Net::Ping->VERSION gt '2.13') {
         $_conf = prompt($prompts{auto_pick}, "yes");
+    } else {
+        prompt("Autoselection disabled due to Net::Ping missing or insufficient. Please press ENTER");
     }
     my @old_list = @{ $CPAN::Config->{urllist} };
     if ( $_conf =~ /^y/i ) {
@@ -1851,8 +1868,7 @@ sub auto_mirrored_by {
     my $local = shift or return;
     local $|=1;
     $CPAN::Frontend->myprint("Looking for CPAN mirrors near you (please be patient)\n");
-    my $mirrors = CPAN::Mirrors->new;
-    $mirrors->parse_mirrored_by($local);
+    my $mirrors = CPAN::Mirrors->new($local);
 
     my $cnt = 0;
     my @best = $mirrors->best_mirrors(
