@@ -3,9 +3,8 @@ package Module::Build::PPMMaker;
 use strict;
 use Config;
 use vars qw($VERSION);
-use IO::File;
 
-$VERSION = '0.4005';
+$VERSION = '0.4202';
 $VERSION = eval $VERSION;
 
 # This code is mostly borrowed from ExtUtils::MM_Unix 6.10_03, with a
@@ -100,12 +99,11 @@ EOF
 EOF
 
   my $ppd_file = "$dist{name}.ppd";
-  my $fh = IO::File->new(">$ppd_file")
+  open(my $fh, '>', $ppd_file)
     or die "Cannot write to $ppd_file: $!";
 
-  my $io_file_ok = eval { IO::File->VERSION(1.13); 1 };
-  $fh->binmode(":utf8")
-    if $io_file_ok && $fh->can('binmode') && $] >= 5.008 && $Config{useperlio};
+  binmode($fh, ":utf8")
+    if $] >= 5.008 && $Config{useperlio};
   print $fh $ppd;
   close $fh;
 
