@@ -33,11 +33,7 @@
  * Important ones in the first cache line (if alignment is done right) */
 
 PERLVAR(I, stack_sp,	SV **)		/* top of the stack */
-#ifdef OP_IN_REGISTER
-PERLVAR(I, opsave,	OP *)
-#else
 PERLVAR(I, op,		OP *)		/* currently executing op */
-#endif
 PERLVAR(I, curpad,	SV **)		/* active pad (lexicals+tmps) */
 
 PERLVAR(I, stack_base,	SV **)
@@ -53,9 +49,9 @@ PERLVAR(I, scopestack_ix, I32)
 PERLVAR(I, scopestack_max, I32)
 
 PERLVAR(I, tmps_stack,	SV **)		/* mortals we've made */
-PERLVARI(I, tmps_ix,	I32,	-1)
-PERLVARI(I, tmps_floor,	I32,	-1)
-PERLVAR(I, tmps_max,	I32)
+PERLVARI(I, tmps_ix,	SSize_t,	-1)
+PERLVARI(I, tmps_floor,	SSize_t,	-1)
+PERLVAR(I, tmps_max,	SSize_t)
 
 PERLVARI(I, sub_generation, U32, 1)	/* incr to invalidate method cache */
 
@@ -540,8 +536,6 @@ PERLVAR(I, padix,	I32)		/* max used index in current "register" pad */
 
 PERLVAR(I, padix_floor,	I32)		/* how low may inner block reset padix */
 
-PERLVAR(I, hints,	U32)		/* pragma-tic compile-time flags */
-
 #ifdef USE_LOCALE_COLLATE
 PERLVAR(I, collation_name, char *)	/* Name of current collation */
 PERLVAR(I, collxfrm_base, Size_t)	/* Basic overhead in *xfrm() */
@@ -582,8 +576,8 @@ PERLVAR(I, numeric_radix_sv, SV *)	/* The radix separator if not '.' */
 #endif /* !USE_LOCALE_NUMERIC */
 
 /* Unicode inversion lists */
-PERLVAR(I, ASCII,	SV *)
 PERLVAR(I, Latin1,	SV *)
+PERLVAR(I, UpperLatin1,	SV *)   /* Code points 128 - 255 */
 PERLVAR(I, AboveLatin1,	SV *)
 
 PERLVAR(I, NonL1NonFinalFold,   SV *)
@@ -741,7 +735,7 @@ PERLVAR(I, debug_pad,	struct perl_debug_pad)	/* always needed because of the re
 /* Hook for File::Glob */
 PERLVARI(I, globhook,	globhook_t, NULL)
 
-/* The last unconditional member of the interpreter structure when 5.19.2 was
+/* The last unconditional member of the interpreter structure when 5.19.6 was
    released. The offset of the end of this is baked into a global variable in 
    any shared perl library which will allow a sanity test in future perl
    releases.  */
@@ -786,6 +780,8 @@ PERLVARA(I, op_exec_cnt, OP_max+2, UV)	/* Counts of executed OPs of the given ty
                                            DEBUGGING is enabled, too. */
 #endif
 
+PERLVAR(I, random_state, PL_RANDOM_STATE_TYPE)
+
 /* If you are adding a U8 or U16, check to see if there are 'Space' comments
  * above on where there are gaps which currently will be structure padding.  */
 
