@@ -489,7 +489,6 @@ db_prefixtype='size_t'
 db_version_major='0'
 db_version_minor='0'
 db_version_patch='0'
-defvoidused=1
 direntrytype='struct dirent'
 doublesize='8'
 drand01="((rand() & 0x7FFF) / (double) ((unsigned long)1 << 15))"
@@ -554,7 +553,6 @@ i_crypt='undef'
 i_db='undef'
 i_dbm='undef'
 i_dirent='define'
-i_dld='undef'
 i_dlfcn='undef'
 i_fcntl='undef'
 i_float='undef'
@@ -818,5 +816,4 @@ vendorarchexp=''
 vendorlib_stem=''
 vendorlibexp=''
 versiononly='undef'
-voidflags=1
 zip=''
